import plugin_util as PU 
import httplib
import datetime
import urllib

from com.xhaus.jyson import JysonCodec as json

OPENBIS_SERVERS_DIR = "/Users/hullarb/Work/workspace/openbis/servers"
WORKFLOW_FILE = OPENBIS_SERVERS_DIR + "/core-plugins/beewm/1/workflows/bee_demo_analysis.xml"
DATASET_ID_KEY = "api.datasetId" 
PARAMETER_KEY = "api.toolparam"
EMAIL_KEY = "api.email"
BEE_SERVER = "127.0.0.1:9999"

REQUEST_HEADER = {"Content-type": "application/x-www-form-urlencoded", "Accept": "text/plain"}

def aggregate(parameters, tableBuilder):
    '''Main entry point for aggregation services'''    
    PU.init_result_table(tableBuilder)
    if not authorized(parameters):
        PU.create_status_reply(PU.FAILED, "Not authorized", tableBuilder)
        return

    if PU.get_method(parameters) == "submit":
        submit(parameters, tableBuilder)

    if PU.get_method(parameters) == "getStatus":
        get_status(parameters, tableBuilder)

def authorized(parameters):
    permid = PU.get_permid(parameters)
    ds = searchService.getDataSet(permid)
    print ds
    return ds

def process(tr, parameters, tableBuilder):
    '''
    Main entry point of the plugin
    '''
    aggregate(parameters, tableBuilder) # delegate to aggregate function

def submit(parameters, tableBuilder):
    '''
    Submits a processing
    '''
    print "Start processing on dataset " + PU.get_permid(parameters) + " by " + PU.get_userid(parameters)
    success = submit_new_processing(WORKFLOW_FILE, parameters)
    if success:
        PU.create_status_reply(PU.SUCCESSFUL, "Submission was successful.", tableBuilder)
    else:
        PU.create_status_reply(PU.FAILED, "Submission failed", tableBuilder)        

def submit_new_processing(workflow_file, parameters):
    params = prepare_submission_parameters(workflow_file, parameters)
    conn = httplib.HTTPConnection(BEE_SERVER)
    conn.request("POST", "/apiv1/processes", params, REQUEST_HEADER)
    resp = conn.getresponse()
    return resp.status == 200

def prepare_submission_parameters(workflow_file, parameters):
    workflow = open(workflow_file).read()
    permid = PU.get_permid(parameters)
    tool_parameter = PU.get_toolparameter(parameters)
    userId = PU.get_userid(parameters)  
    if not tool_parameter:
        tool_parameter = " " 
    return urllib.urlencode({'workflow': workflow, DATASET_ID_KEY : permid, PARAMETER_KEY : tool_parameter, EMAIL_KEY : userId})

def get_status(parameters, tableBuilder):
    permid = PU.get_permid(parameters)
    raw_status = query_status(permid)
    statusTxt = ""
    if raw_status != 'null':
      data = json.loads(raw_status)
      processes = data['workflowDTO']
    else:
      statusTxt = "No Job started."
      PU.create_status_reply(PU.NO_PROCESS, statusTxt, tableBuilder)
      return
    if not isinstance(processes, list):
      processes = [processes]
    if not statusTxt:
        statusTxt = "Processing history for dataset " + permid +  ": <br/>"
    running = False
    for p in processes:
      d = datetime.datetime.fromtimestamp(float(p['startTime'])/1000.0)
      statusTxt += "Process started: " + str(d) + " is in state: " + p['status'] + "<br/>"
      if not isFinished(p['status']):
        running = True
    if not running:
        PU.create_status_reply(PU.RUNNING, statusTxt, tableBuilder)
    else:
        PU.create_status_reply(PU.NOT_RUNNING, statusTxt, tableBuilder)
    
def query_status(permid):    
    conn = httplib.HTTPConnection(BEE_SERVER)
    conn.request("GET", "/apiv1/processes?inputId=" + permid)
    resp = conn.getresponse().read()
    return resp

def isFinished(status):
   return status == 'COMPLETE' or status == 'ABORTED' or status == 'ERROR' or status == 'WARNING'
